// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include <vector>
#include "Common.h"
#include "AccountsAndPositions.h"
#include "AccountsAndPositionsDlg.h"

void CAccountsAndPositionsDlg::InitializeCQGCEL()
{
   // create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   AssertCOMError(m_spCQGCEL, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   AssertCOMError(spConf, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_FireEventOnChangedPrices(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_PositionDetailing(pdAllTrades);
   AssertCOMError(spConf, hr);

   hr = spConf->put_PriceMode(pmTradesOnly);
   AssertCOMError(spConf, hr);

   hr = spConf->put_DefPositionSubscriptionLevel(pslSnapshotAndUpdates);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UseOrderSide(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_DefaultAccountMarginDetailing(amdNoMargin);
   AssertCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   assert(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   AssertCOMError(m_spCQGCEL, hr);
}

void CAccountsAndPositionsDlg::InitializeControls()
{
   OnDataConnectionStatusChanged(csConnectionDown);
   OnGWConnectionStatusChanged(csConnectionDown);

   InitializeReportDays();
   InitializeAccProperties();
   InitializeAccSummary();
   InitializePositions();

   ClearAccountData();
}

void CAccountsAndPositionsDlg::InitializeAccProperties()
{
   m_lvAccProperties.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvAccProperties.InsertColumn(apcName, _T("Name"), LVCFMT_LEFT, 135);
   m_lvAccProperties.InsertColumn(apcValue, _T("Value"), LVCFMT_LEFT, 130);

   // Add empty rows to the list
   for (int i = 0; i < aprPropertiesRowsCount; ++i)
   {
      m_lvAccProperties.InsertItem(apcName, _T(""));
   }

   // Fill property names to list
   m_lvAccProperties.SetItemText(aprFCMID, apcName, _T("FCM ID"));
   m_lvAccProperties.SetItemText(aprFCMAccountID, apcName, _T("FCM Account ID"));
   m_lvAccProperties.SetItemText(aprFCMName, apcName, _T("FCM Name"));
   m_lvAccProperties.SetItemText(aprGWAccountName, apcName, _T("GW Account Name"));
   m_lvAccProperties.SetItemText(aprGWAccountID, apcName, _T("GW Account ID"));
   m_lvAccProperties.SetItemText(aprDateOfLastStatement, apcName, _T("Date of Last Statement"));
   m_lvAccProperties.SetItemText(aprReportingCurrency, apcName, _T("Reporting Currency"));
   m_lvAccProperties.SetItemText(aprTimestampOfLastUpdate, apcName, _T("Timestamp of Last Update"));
   m_lvAccProperties.SetItemText(aprTotalMargin, apcName, _T("Total Margin"));
   m_lvAccProperties.SetItemText(aprTotalPurchasingPower, apcName, _T("Total Purchasing Power"));
   m_lvAccProperties.SetItemText(aprMarginDetailingLevel, apcName, _T("Margin Detailing Level"));
}

void CAccountsAndPositionsDlg::InitializeAccSummary()
{
   m_lvAccSummary.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvAccSummary.InsertColumn(ascName, _T("Name"), LVCFMT_LEFT, 203);
   m_lvAccSummary.InsertColumn(ascRepCur, _T(""), LVCFMT_LEFT, m_stSummaryColWidth);
   m_lvAccSummary.InsertColumn(ascCur, _T(""), LVCFMT_LEFT, m_stSummaryColWidth);

    // Add empty rows to the list
   for (int i = 0; i < asrSummaryRowsCount; ++i)
   {
      m_lvAccSummary.InsertItem(ascName, _T(""));
   }

   // Fill property names to the list
   m_lvAccSummary.SetItemText(asrBalance, ascName, _T("Account Balance"));
   m_lvAccSummary.SetItemText(asrPL, ascName, _T("Profit/Loss"));
   m_lvAccSummary.SetItemText(asrOTE, ascName, _T("Open Trade Equity for Futures"));
   m_lvAccSummary.SetItemText(asrUPL, ascName, _T("Unrealized Profit/Loss for Options"));
   m_lvAccSummary.SetItemText(asrCollaterals, ascName, _T("Collateral on Deposit"));
   m_lvAccSummary.SetItemText(asrNLV, ascName, _T("Net Liquidity Value"));
   m_lvAccSummary.SetItemText(asrMVO, ascName, _T("Market Value of Options"));
   m_lvAccSummary.SetItemText(asrCashExcess, ascName, _T("Cash Excess"));
   m_lvAccSummary.SetItemText(asrInitialMargin, ascName, _T("Initial Margin"));
   m_lvAccSummary.SetItemText(asrMaintenanceMargin, ascName, _T("Maintenance Margin"));
}

void CAccountsAndPositionsDlg::InitializePositions()
{
   m_lvPositions.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

   m_lvPositions.InsertColumn(pcInstrument, _T("Instrument"), LVCFMT_LEFT, 115);
   m_lvPositions.InsertColumn(pcLong, _T("Long"), LVCFMT_LEFT, 65);
   m_lvPositions.InsertColumn(pcShort, _T("Short"), LVCFMT_LEFT, 65);
   m_lvPositions.InsertColumn(pcAveragePrice, _T("Average Price"), LVCFMT_LEFT, 85);
   m_lvPositions.InsertColumn(pcOTE, _T("OTE"), LVCFMT_LEFT, 85);
   m_lvPositions.InsertColumn(pcMVO, _T("MVO"), LVCFMT_LEFT, 85);
   m_lvPositions.InsertColumn(pcOffset, _T("Offset"), LVCFMT_LEFT, 80);
   m_lvPositions.InsertColumn(pcPL, _T("Profit/Loss"), LVCFMT_LEFT, 80);
   m_lvPositions.InsertColumn(pcCurrency, _T("Currency"), LVCFMT_LEFT, 80);
}

void CAccountsAndPositionsDlg::InitializeReportDays()
{
   // Add names of report days to the combo and match items with their days. 
   m_cmbReportDays.InsertString(0, _T("Today"));
   m_cmbReportDays.SetItemData(0, 0);

   m_cmbReportDays.InsertString(1, _T("Yesterday"));
   m_cmbReportDays.SetItemData(1, 1);
}

void CAccountsAndPositionsDlg::ReloadAccounts()
{
   try
   {
      // Clear all previous data
      m_cmbAccounts.ResetContent();

      ATL::CComPtr<ICQGAccounts> spAccounts;
      HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
      AssertCOMError(m_spCQGCEL, hr);

      long count;
      hr = spAccounts->get_Count(&count);
      AssertCOMError(spAccounts, hr);

      if (count == 0)
      {
         // No accounts
         return;
      }

      // Iterate over accounts collection and add them to list
      ATL::CComPtr<IEnumVARIANT> spEnumVariant;
      hr = spAccounts->get__NewEnum(&spEnumVariant);
      AssertCOMError(spAccounts, hr);

      std::vector<ATL::CComVariant> pItems(count);
      ULONG celtFetched;
      hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
      AssertCOMError(spEnumVariant, hr);

      for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
      {
         assert(it->vt == VT_DISPATCH);

         ATL::CComPtr<ICQGAccount> spAccount;
         hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
         AssertCOMError<IDispatch>(it->pdispVal, hr);

         // Requests subscription for each instrument on 
         // which the current account has a position
         spAccount->put_AutoSubscribeInstruments(VARIANT_TRUE);

         // Requests positions and their updates
         spAccount->put_PositionSubcriptionLevel(pslSnapshotAndUpdates);

         // Add current account to list
         InsertAccount(spAccount);
      }

      // Select the first account in the list
      if (m_cmbAccounts.GetCount() > 0)
      {
         m_cmbAccounts.SetCurSel(0);
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CAccountsAndPositionsDlg::InsertAccount(
                                    const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Create string for specified account to add to the list
   ATL::CComBSTR accName;
   HRESULT hr = spAccount->get_GWAccountName(&accName);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR FcmAccID;
   hr = spAccount->get_FcmAccountID(&FcmAccID);
   AssertCOMError(spAccount, hr);

   CString accString = ATL::CW2T(accName);
   accString = accString + _T(" (") + ATL::CW2T(FcmAccID) + _T(")");

   // Get account ID to correspond with item
   long accID;
   hr = spAccount->get_GWAccountID(&accID);
   AssertCOMError(spAccount, hr);

   // Get last row to insert new account name to the end 
   int row = m_cmbAccounts.GetCount();

   m_cmbAccounts.InsertString(row, accString);
   m_cmbAccounts.SetItemData(row, accID);
}

void CAccountsAndPositionsDlg::FillCurrencies(
                                    const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Clear old data
   m_cmbCurrencies.ResetContent();

   ATL::CComPtr<ICQGAccountSummaries> spSummaries;
   HRESULT hr = spAccount->get_CurrencySummaries(&spSummaries);
   AssertCOMError(spAccount, hr);

   long count;
   hr = spSummaries->get_Count(&count);
   AssertCOMError(spSummaries, hr);

   // Iterate over summary reports for all currencies and 
   // add all currencies to combo
   ATL::CComPtr<ICQGAccountSummary> spSummary;
   for (int i = 0; i < count; ++i)
   {
      hr = spSummaries->get_Item(ATL::CComVariant(i), &spSummary);
      AssertCOMError(spSummaries, hr);

      ATL::CComBSTR currency;
      spSummary->get_CurrencyName(&currency);
      AssertCOMError(spSummary, hr);

      m_cmbCurrencies.InsertString(0, ATL::CW2T(currency));

      spSummary.Release();
   }
   
   if (m_cmbCurrencies.GetCount() > 0)
   {
      m_cmbCurrencies.SetCurSel(0);
   }
}

ATL::CComPtr<ICQGAccount> CAccountsAndPositionsDlg::GetSelectedAccount() const
{
   ATL::CComPtr<ICQGAccounts> spAccounts;
   HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
   AssertCOMError(m_spCQGCEL, hr);

   long accID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

   ATL::CComPtr<ICQGAccount> spAccount;
   hr = spAccounts->get_Item(accID, &spAccount);
   AssertCOMError(spAccounts, hr);

   return spAccount;
}

void CAccountsAndPositionsDlg::ShowSelectedAccountData()
{
   // Clear old data
   ClearAccountData();

   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();

   // Fill all available currencies for specified account
   FillCurrencies(spAccount);

   // Show all data
   ShowSummaries(spAccount);
   ShowProperties(spAccount);
   ShowPositions(spAccount);
}

void CAccountsAndPositionsDlg::ShowSummaries(
                                    const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Get reporting day
   int reportDay = static_cast<int>(m_cmbReportDays.GetItemData(m_cmbReportDays.GetCurSel()));

   // Get reporting currency
   CString currency;
   
   // Account can have no summary
   if (m_cmbCurrencies.GetCurSel() > -1)
   {
      m_cmbCurrencies.GetLBText(m_cmbCurrencies.GetCurSel(), currency);

      ShowAccountSummaries(spAccount, currency.GetBuffer(), reportDay);
   }
}

void CAccountsAndPositionsDlg::ClearAccountData()
{
   // Clear all summaries' data
   for (int i = 0; i < asrSummaryRowsCount; ++i)
   {
      m_lvAccSummary.SetItemText(i, ascRepCur, m_N_A);
      m_lvAccSummary.SetItemText(i, ascCur, m_N_A);
   }

   // Clear headers
   LVCOLUMN col = { 0 };
   m_lvAccSummary.GetColumn(ascRepCur, &col);
   col.pszText = _T("Total");
   m_lvAccSummary.SetColumn(ascRepCur, &col);

   m_lvAccSummary.GetColumn(ascCur, &col);
   col.pszText = _T("");
   m_lvAccSummary.SetColumn(ascCur, &col);

   // Delete all positions
   m_lvPositions.DeleteAllItems();

   // Clears account properties
   for (int i = 0; i < aprPropertiesRowsCount; ++i)
   {
      m_lvAccProperties.SetItemText(i, apcValue, m_N_A);
   }

   // Reset report days combo
   m_cmbReportDays.SetCurSel(0);
}

bool CAccountsAndPositionsDlg::IsAccountSelected(
                                    const ATL::CComPtr<ICQGAccount>& spAccount) const
{
   // Get account ID 
   long chgAccID;
   HRESULT hr = spAccount->get_GWAccountID(&chgAccID);
   AssertCOMError(spAccount, hr);

   // Get selected account ID
   long selAccID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

   return (chgAccID == selAccID);
}

void CAccountsAndPositionsDlg::ShowAccountSummaries(
                                    const ATL::CComPtr<ICQGAccount>& spAccount, 
                                    ATL::CComBSTR currency, 
                                    int reportDay)
{
   // Show account summary which is calculated based on 
   // the account's existing summaries for different currencies, 
   // converted to the accounts reporting currency
   ATL::CComPtr<ICQGAccountSummary> spAccSummary;
   HRESULT hr = spAccount->get_Summary(&spAccSummary);
   AssertCOMError(spAccount, hr);

   // Add column for this summary
   // The first column is for account summary properties names,
   ATL::CComBSTR repCurrName;
   hr = spAccount->get_ReportingCurrency(&repCurrName);
   AssertCOMError(spAccount, hr);

   CString header = _T("Total (");
   header = header + ATL::CW2T(repCurrName) + _T(")");

   // Change column header
   LVCOLUMN col = { 0 };
   m_lvAccSummary.GetColumn(ascRepCur, &col);
   col.pszText = header.GetBuffer();
   m_lvAccSummary.SetColumn(ascRepCur, &col);

   ShowAccountSummary(spAccSummary, reportDay, ascRepCur);
   spAccSummary.Release();

   // Show account summaries for selected currency
   ATL::CComPtr<ICQGAccountSummaries> spAccSummaries;
   hr = spAccount->get_CurrencySummaries(&spAccSummaries);
   AssertCOMError(spAccount, hr);

   hr = spAccSummaries->get_Item(ATL::CComVariant(currency), &spAccSummary);
   AssertCOMError(spAccSummaries, hr);

   if (!spAccSummary) return;

   ATL::CComBSTR currName;
   hr = spAccSummary->get_CurrencyName(&currName);
   AssertCOMError(spAccSummary, hr);

   // Change column header
   m_lvAccSummary.GetColumn(ascCur, &col);
   header = ATL::CW2T(currName);
   col.pszText = header.GetBuffer();
   m_lvAccSummary.SetColumn(ascCur, &col);

   // Fill summary values
   ShowAccountSummary(spAccSummary, reportDay, ascCur);
}

void CAccountsAndPositionsDlg::ShowAccountSummary(
                                    const ATL::CComPtr<ICQGAccountSummary>& spAccSummary, 
                                    int reportDay, 
                                    int column)
{
   double accBalance;
   HRESULT hr = spAccSummary->Balance(reportDay, &accBalance);
   AssertCOMError(spAccSummary, hr);

   double accOTE;
   hr = spAccSummary->OTE(reportDay, &accOTE);
   AssertCOMError(spAccSummary, hr);

   double accUPL;
   hr = spAccSummary->UPL(reportDay, &accUPL);
   AssertCOMError(spAccSummary, hr);

   double accNLV;
   hr = spAccSummary->NLV(reportDay, &accNLV);
   AssertCOMError(spAccSummary, hr);

   double accMVO;
   hr = spAccSummary->MVO(reportDay, &accMVO);
   AssertCOMError(spAccSummary, hr);

   m_lvAccSummary.SetItemText(asrBalance, column, GetValueAsString(accBalance));
   m_lvAccSummary.SetItemText(asrOTE, column, GetValueAsString(accOTE));
   m_lvAccSummary.SetItemText(asrUPL, column, GetValueAsString(accUPL));
   m_lvAccSummary.SetItemText(asrNLV, column, GetValueAsString(accNLV));
   m_lvAccSummary.SetItemText(asrMVO, column, GetValueAsString(accMVO));

   // In this release only one report day back (i.e., "yesterday close") value is available
   // for the following values
   if (reportDay == 0) 
   {
      double accCollaterals;
      hr = spAccSummary->Collaterals(1, &accCollaterals);
      AssertCOMError(spAccSummary, hr);

      double accCashExcess;
      hr = spAccSummary->CashExcess(1, &accCashExcess);
      AssertCOMError(spAccSummary, hr);

      double accInitialMargin;
      hr = spAccSummary->InitialMargin(1, &accInitialMargin);
      AssertCOMError(spAccSummary, hr);

      double accMaintenanceMargin;
      hr = spAccSummary->MaintenanceMargin(1, &accMaintenanceMargin);
      AssertCOMError(spAccSummary, hr);

      double accPrevBalance;
      hr = spAccSummary->Balance(1 , &accPrevBalance);
      AssertCOMError(spAccSummary, hr);

      // Profit/Loss is calculated from current and previous balances
      VARIANT_BOOL isValid, isValidPrev;
      hr = m_spCQGCEL->IsValid(ATL::CComVariant(accBalance), &isValid);
      AssertCOMError(m_spCQGCEL, hr);

      hr = m_spCQGCEL->IsValid(ATL::CComVariant(accPrevBalance), &isValidPrev);
      AssertCOMError(m_spCQGCEL, hr);
      
      CString profitLossStr = isValid && isValidPrev ? 
                              VariantToString(accBalance - accPrevBalance) : 
                              m_N_A;

      m_lvAccSummary.SetItemText(asrPL, column, profitLossStr);
      m_lvAccSummary.SetItemText(asrCollaterals, column, GetValueAsString(accCollaterals));
      m_lvAccSummary.SetItemText(asrCashExcess, column, GetValueAsString(accCashExcess));
      m_lvAccSummary.SetItemText(asrInitialMargin, column, GetValueAsString(accInitialMargin));
      m_lvAccSummary.SetItemText(asrMaintenanceMargin, column, GetValueAsString(accMaintenanceMargin));
   }
   else
   {
      m_lvAccSummary.SetItemText(asrPL, column, m_N_A);
      m_lvAccSummary.SetItemText(asrCollaterals, column, m_N_A);
      m_lvAccSummary.SetItemText(asrCashExcess, column, m_N_A);
      m_lvAccSummary.SetItemText(asrInitialMargin, column, m_N_A);
      m_lvAccSummary.SetItemText(asrMaintenanceMargin, column, m_N_A);
   }
}

void CAccountsAndPositionsDlg::ShowPositions(
                                    const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Clear all previous positions
   m_lvPositions.DeleteAllItems();   

   ATL::CComPtr<ICQGPositions> spPositions;
   HRESULT hr = spAccount->get_Positions(&spPositions);
   AssertCOMError(spAccount, hr);

   long count;
   hr = spPositions->get_Count(&count);
   AssertCOMError(spPositions, hr);

   if (count == 0)
   {
      // No positions
      return;
   }

   // Iterate over positions collection and add them to list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spPositions->get__NewEnum(&spEnumVariant);
   AssertCOMError(spPositions, hr);

   std::vector<ATL::CComVariant> pItems(count);
   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      assert(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGPosition> spPosition;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGPosition), (void** )&spPosition);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current position to list
      ShowPosition(spPosition);
   }
}

void CAccountsAndPositionsDlg::ShowPosition(
                                    const ATL::CComPtr<ICQGPosition>& spPosition)
{
   ATL::CComBSTR fullName;
   HRESULT hr = spPosition->get_InstrumentName(&fullName);
   AssertCOMError(spPosition, hr);

   // Find row for specified position. Add new one if not found
   CString fullNameCStr = ATL::CW2T(fullName);
   LVFINDINFO findInfo;
   findInfo.flags = LVFI_STRING;
   findInfo.psz = fullNameCStr;
   int row = m_lvPositions.FindItem(&findInfo);

   if (row == -1)
   {
      row = m_lvPositions.InsertItem(m_lvPositions.GetItemCount(), _T(""));
   }

   // Show values for specified positions
   double averagePrice;
   hr = spPosition->get_AveragePrice(&averagePrice);
   AssertCOMError(spPosition, hr);

   double MVO;
   hr = spPosition->get_MVO(&MVO);
   AssertCOMError(spPosition, hr);

   double OTE;
   hr = spPosition->get_OTE(&OTE);
   AssertCOMError(spPosition, hr);

   double profitLoss;
   hr = spPosition->get_ProfitLoss(&profitLoss);
   AssertCOMError(spPosition, hr);

   m_lvPositions.SetItemText(row, pcInstrument, fullNameCStr);   
   m_lvPositions.SetItemText(row, pcMVO, GetValueAsString(MVO));
   m_lvPositions.SetItemText(row, pcOTE, GetValueAsString(OTE));
   m_lvPositions.SetItemText(row, pcPL, GetValueAsString(profitLoss));

   // Get position instrument
   ATL::CComPtr<ICQGInstrument> spInstrument;
   hr = spPosition->get_Instrument(&spInstrument);
   AssertCOMError(spPosition, hr);

   // Convert to display price and show currency if instrument is subscribed
   if (spInstrument == NULL)
   {
      m_lvPositions.SetItemText(row, pcAveragePrice, m_N_A);
      m_lvPositions.SetItemText(row, pcCurrency, m_N_A);
   }
   else
   {
      ATL::CComBSTR dispPrice;
      hr = spInstrument->ToDisplayPrice(averagePrice, &dispPrice);
      AssertCOMError(spInstrument, hr);

      ATL::CComBSTR currency;
      hr = spInstrument->get_Currency(&currency);
      AssertCOMError(spInstrument, hr);

      m_lvPositions.SetItemText(row, pcAveragePrice, ATL::CW2T(dispPrice));
      m_lvPositions.SetItemText(row, pcCurrency, ATL::CW2T(currency));
   }

   // Calculate offset from matched trades
   ATL::CComPtr<ICQGTrades> spTrades;
   hr = spPosition->get_MatchedTrades(&spTrades);
   AssertCOMError(spPosition, hr);

   if (spTrades != NULL)
   {
      long count;
      hr = spTrades->get_Count(&count);
      AssertCOMError(spTrades, hr);

      long offset = 0;
      for (long i = 0; i < count; ++i)
      {
         ATL::CComPtr<ICQGTrade> spTrade;
         hr = spTrades->get_Item(i, &spTrade);
         AssertCOMError(spTrades, hr);

         long quantity;
         hr = spTrade->get_Quantity(&quantity);
         AssertCOMError(spTrade, hr);

         offset += quantity;
      }
      m_lvPositions.SetItemText(row, pcOffset, GetValueAsString(offset));
   }
   else
   {
      m_lvPositions.SetItemText(row, pcOffset, m_N_A);
   }

   // Calculate long/short based on position side
   eOrderSide side;
   hr = spPosition->get_Side(&side);
   AssertCOMError(spPosition, hr);

   long quantity;
   hr = spPosition->get_Quantity(&quantity);
   AssertCOMError(spPosition, hr);
   
   VARIANT_BOOL isValid;
   hr = m_spCQGCEL->IsValid(ATL::CComVariant(quantity), &isValid);
   AssertCOMError(spPosition, hr); 

   if (isValid == VARIANT_TRUE)
   {
      switch (side)
      {
         case osdBuy:
            m_lvPositions.SetItemText(row, pcLong, VariantToString(quantity));
            m_lvPositions.SetItemText(row, pcShort, _T("0"));
            break;
         case osdSell:
            m_lvPositions.SetItemText(row, pcLong, _T("0"));
            m_lvPositions.SetItemText(row, pcShort, VariantToString(quantity));
            break;
         default:
            // offset-only case, still need to show row
            assert(quantity == 0);
            m_lvPositions.SetItemText(row, pcLong, _T("0"));
            m_lvPositions.SetItemText(row, pcShort, _T("0"));
      }
   }
   else
   {
      m_lvPositions.SetItemText(row, pcLong, m_N_A);
      m_lvPositions.SetItemText(row, pcShort, m_N_A);
   }
}

void CAccountsAndPositionsDlg::ShowProperties(
                                    const ATL::CComPtr<ICQGAccount>& spAccount)
{
   long fcmID;
   HRESULT hr = spAccount->get_FcmID(&fcmID);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR fcmAccountID;
   hr = spAccount->get_FcmAccountID(&fcmAccountID);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR fcmName;
   hr = spAccount->get_FcmName(&fcmName);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR gwAccountName;
   hr = spAccount->get_GWAccountName(&gwAccountName);
   AssertCOMError(spAccount, hr);

   long gwAccountID;
   hr = spAccount->get_GWAccountID(&gwAccountID);
   AssertCOMError(spAccount, hr);

   DATE dateOfLastStatement;
   hr = spAccount->get_DateOfLastStatement(&dateOfLastStatement);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR reportingCurrency;
   hr = spAccount->get_ReportingCurrency(&reportingCurrency);
   AssertCOMError(spAccount, hr);

   DATE timestamp;
   hr = spAccount->get_Timestamp(&timestamp);
   AssertCOMError(spAccount, hr);

   DOUBLE totalMargin;
   hr = spAccount->get_TotalMarginValue(&totalMargin);
   AssertCOMError(spAccount, hr);

   DOUBLE totalPurchasePower;
   hr = spAccount->get_TotalPurchasingPower(&totalPurchasePower);
   AssertCOMError(spAccount, hr);

   eAccountMarginDetailing marginDetailingLevel;
   hr = spAccount->get_AccountMarginDetailing(&marginDetailingLevel);
   AssertCOMError(spAccount, hr);

   m_lvAccProperties.SetItemText(aprFCMID, apcValue, GetValueAsString(fcmID));
   m_lvAccProperties.SetItemText(aprFCMAccountID, apcValue, GetValueAsString(fcmAccountID));
   m_lvAccProperties.SetItemText(aprFCMName, apcValue, GetValueAsString(fcmName));
   m_lvAccProperties.SetItemText(aprGWAccountName, apcValue, GetValueAsString(gwAccountName));
   m_lvAccProperties.SetItemText(aprGWAccountID, apcValue, GetValueAsString(gwAccountID));
   m_lvAccProperties.SetItemText(aprReportingCurrency, apcValue, GetValueAsString(reportingCurrency));
   m_lvAccProperties.SetItemText(aprTimestampOfLastUpdate, apcValue,
                                             GetValueAsString(ATL::CComVariant(timestamp, VT_DATE)));
   m_lvAccProperties.SetItemText(aprTotalMargin, apcValue, GetValueAsString(totalMargin));
   m_lvAccProperties.SetItemText(aprTotalPurchasingPower, apcValue, GetValueAsString(totalPurchasePower));
   m_lvAccProperties.SetItemText(aprMarginDetailingLevel, apcValue, AccountMarginDetailingToString(marginDetailingLevel));
  
   VARIANT_BOOL isValid;
   hr = m_spCQGCEL->IsValid(ATL::CComVariant(dateOfLastStatement, VT_DATE), &isValid);
   AssertCOMError(m_spCQGCEL, hr);
   if (isValid)
   {
      // Only date part is showed
      m_lvAccProperties.SetItemText(aprDateOfLastStatement, apcValue, 
                                    DateTimeToString(dateOfLastStatement, false));
   }
   else
   {
      m_lvAccProperties.SetItemText(aprDateOfLastStatement, apcValue, m_N_A);
   }
}

CString CAccountsAndPositionsDlg::GetValueAsString(
                                    const ATL::CComVariant& value) const
{

   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return VariantToString(value);
   }

   return m_N_A;
}

void CAccountsAndPositionsDlg::EnableControls(
                                    bool enable)
{
   m_cmbAccounts.EnableWindow(enable);
   m_cmbReportDays.EnableWindow(enable);
   m_cmbCurrencies.EnableWindow(enable);
}

CString CAccountsAndPositionsDlg::ConnectionStatusToString(
                                    eConnectionStatus status)
{
   CString  statusString;

   switch (status)
   {
   case csConnectionDelayed:
      statusString = _T("delayed");
      break;
   case csConnectionDown:
      statusString = _T("down");
      break;
   case csConnectionUp:
      statusString = _T("up");
      break;
   case csConnectionTrouble:
      statusString = _T("trouble");
      break;
   case csConnectionNotLoggedOn:
      statusString = _T("not logged on");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return statusString;
}

CString CAccountsAndPositionsDlg::AccountMarginDetailingToString(eAccountMarginDetailing detailingValue)
{
   switch (detailingValue)
   {
      case amdNoMargin:
         return _T("NoMargin");
      case amdMarginSnapshot:
         return _T("MarginSnapshot");
      case amdMarginSnapshotAndUpdate:
         return _T("MarginSnapshotAndUpdate");
      default:
         return m_N_A;
   }
}
