// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "AccountsAndPositions.h"
#include "AccountsAndPositionsDlg.h"

namespace
{
   /// \brief Shows message box with symbol resolution failed reason.
   /// \param symbol - Requested symbol
   /// \param cqg_error - Valid when error occured
   void onSymbolResolutonFailed(BSTR symbol, ICQGError* cqg_error)
   {
      ATL::CComBSTR errDescription;
      auto hr = cqg_error->get_Description(&errDescription);
      AssertCOMError<ICQGError>(cqg_error, hr);

      CStringW msg;
      msg.AppendFormat(L"%s resolution failed. %s", symbol, errDescription.m_str);
      AfxMessageBox(ATL::CW2T(msg), MB_ICONWARNING);
   }
}

STDMETHODIMP CAccountsAndPositionsDlg::OnDataError(
                                 LPDISPATCH /*obj*/,
                                 BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CAccountsAndPositionsDlg::OnGWConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   m_lblGWConnection.SetWindowText(_T("GW connection : ") + ConnectionStatusToString(newStatus));

   if (newStatus == csConnectionUp)
   {
      m_spCQGCEL->put_AccountSubscriptionLevel(aslAccountsAndUpdates);
   }

   // Disable controls until accounts are reloaded
   EnableControls(false);

   return S_OK;
}

STDMETHODIMP CAccountsAndPositionsDlg::OnDataConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   m_lblDataConnection.SetWindowText(_T("Data connection : ") + ConnectionStatusToString(newStatus));

   return S_OK;
}

STDMETHODIMP CAccountsAndPositionsDlg::OnAccountChanged(
                                 eAccountChangeType change, 
                                 ICQGAccount* account, 
                                 ICQGPosition* position)
{
   try
   {
      switch (change)
      {
         case actAccountsReloaded:
            ReloadAccounts();
            EnableControls(true);
            ShowSelectedAccountData();
            break;
         case actAccountChanged:
            if (IsAccountSelected(account))
            {
               ShowProperties(account);
            }
            break;
         case actPositionAdded:
         case actPositionChanged:
            if (IsAccountSelected(account))
            {
               ShowSummaries(account);
               ShowPosition(position);
            }
            break;
         case actPositionsReloaded:
            if (IsAccountSelected(account))
            {
               ShowSummaries(account);
               ShowPositions(account);
            }
            break;
         default:
            assert(!"Unexpected account change type");
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAccountsAndPositionsDlg::OnInstrumentResolved(
   BSTR symbol, ICQGInstrument* instrument, ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL errorOccured = VARIANT_FALSE;
      auto hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &errorOccured);
      AssertCOMError<ICQGError>(cqg_error, hr);

      if (errorOccured)
      {
         onSymbolResolutonFailed(symbol, cqg_error);
         return S_OK;
      }

      ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();
      if (spAccount == NULL)
      {
         // No selected account
         return S_OK;
      }

      ShowSummaries(spAccount);

      // Get position for this instrument if exists
      ATL::CComPtr<ICQGPositions> spPositions;
      hr = spAccount->get_Positions(&spPositions);
      AssertCOMError(spAccount, hr);

      ATL::CComBSTR fullName;
      hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      ATL::CComPtr<ICQGPosition> spPosition;
      hr = spPositions->get_Item(fullName, &spPosition);
      AssertCOMError(spPositions, hr);

      if (spPosition != NULL)
      {
         ShowPosition(spPosition);
      }
   }
   catch (std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}
